package ru.landgrafhomyak.db.serdha0.api.misc

import kotlin.jvm.JvmField
import kotlin.jvm.JvmName
import ru.landgrafhomyak.db.serdha0.api.table.RowId
import ru.landgrafhomyak.db.serdha0.api.table.Table

/**
 * Descriptor of a database type.
 *
 * @param RT Type in programming language to which database type is converted.
 */
@Suppress("ClassName", "RemoveRedundantQualifierName")
public interface DatabaseType<@Suppress("unused") RT> {
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("name")
	public val name: String

	public abstract class _VirtualType<RT, wRT, wDT : DatabaseType<wRT>>(
		@JvmField
		public val wraps: wDT,
		@JvmField
		public val allowValuesCaching: Boolean = true
	) : DatabaseType<RT> {
		@Suppress("FunctionName")
		public abstract fun _unwrap(w: wRT): RT

		@Suppress("FunctionName")
		public abstract fun _wrap(w: RT): wRT

		@Suppress("FunctionName")
		public abstract fun _equals(l: RT, r: RT): Boolean

		@Suppress("FunctionName")
		public abstract fun _compare(l: RT, r: RT): Int
	}

	public interface Nullable<RT : Any, @Suppress("unused") DT : DatabaseType<RT>> : DatabaseType<RT?>

	public interface ROW_ID<ttUE : Any> : DatabaseType<RowId<ttUE>>

	public interface BOOLEAN : DatabaseType<Boolean>

	/**
	 * Descriptor of a raw binary data type for static type-checking.
	 * @see DatabaseType.Provider.BINARY_DATA
	 */
	@OptIn(ExperimentalUnsignedTypes::class)
	public interface BINARY_DATA : DatabaseType<UByteArray> {
		/**
		 * Maximum count of bytes that can be stored by values of this type ot `null` if there is no limit.
		 */
		public val maxSizeOrNull: UInt?

		/**
		 * `true` if value can't hold less than [maxSizeOrNull][DatabaseType.BINARY_DATA.maxSizeOrNull] bytes.
		 */
		public val isFixedSize: Boolean
	}

	public interface Provider {
		@Suppress("INAPPLICABLE_JVM_NAME", "FunctionName")
		@JvmName("ROW_ID")
		public fun <TableUserExtension : Any> ROW_ID(table: Table<TableUserExtension, *>): DatabaseType.ROW_ID<TableUserExtension>

		@Suppress("INAPPLICABLE_JVM_NAME", "PropertyName")
		@get:JvmName("BOOLEAN")
		public val BOOLEAN: BOOLEAN

		@Suppress("INAPPLICABLE_JVM_NAME", "PropertyName")
		@get:JvmName("BINARY_DATA")
		public val BINARY_DATA: BINARY_DATA

		@Suppress("INAPPLICABLE_JVM_NAME")
		@JvmName("BINARY_DATA")
		public fun BINARY_DATA(size: UInt, isFixedSize: Boolean): BINARY_DATA

		public fun <RT : Any, DT : DatabaseType<RT>> nullableOf(notNull: DT): Nullable<RT, DT>

	}
}
