package ru.landgrafhomyak.db.serdha0.api.table

import ru.landgrafhomyak.db.serdha0.api.misc.Column
import ru.landgrafhomyak.db.serdha0.api.misc.DatabaseType
import ru.landgrafhomyak.db.serdha0.api.misc.RowExpression


public interface TableConstructor<tUE : Any> {
	public fun createTable(context: Scope<tUE>): tUE


	public interface Scope<tUE : Any> {
		public val types: DatabaseType.Provider

		public fun <RT, DT : DatabaseType<RT>> column(name: String, type: DT): Column<RT, DT, tUE>

		public fun index(name: String, vararg columns: Column<*, *, tUE>): Index<tUE>

		public fun uniqueIndex(name: String, distinctNulls: Boolean, vararg columns: Column<*, *, tUE>): UniqueIndex<tUE>
		public fun checkConstraint(name: String, constructor: RowExpression.Constructor<tUE, *, Boolean>, recheckExistingRows: Boolean = false): CheckConstraint<tUE>
		public fun <RT : Any, DT : DatabaseType<RT>> defaultValue(c: Column<RT, DT, tUE>, value: RT): ColumnDefaultValue<tUE, RT>
		public fun <RT : Any, DT : DatabaseType<RT>> defaultValue(c: Column<RT, DT, tUE>, creator: RowExpression.Constructor<tUE, *, RT>): ColumnDefaultValue<tUE, RT>

		public val rowIdColumn: Column<RowId<tUE>, DatabaseType.ROW_ID<tUE>, tUE>

		public fun selfRowReference(
			columnName: String,
			onDelete: ForeignRowReference.OnDelete,
		): ForeignRowReference<tUE, tUE, RowId<tUE>, DatabaseType.ROW_ID<tUE>>

		public fun <RT, DT : DatabaseType<RT>> selfRowReference(
			columnName: String,
			type: DT,
			onDelete: ForeignRowReference.OnDelete,
		): ForeignRowReference<tUE, tUE, RT, DT>

		public fun <ttUE : Any> foreignRowReference(
			columnName: String,
			toTable: Table<ttUE, *>,
			onDelete: ForeignRowReference.OnDelete,
		): ForeignRowReference<tUE, ttUE, RowId<ttUE>, DatabaseType.ROW_ID<ttUE>>

		public fun <ttUE : Any, RT, DT : DatabaseType<RT>> foreignRowReference(
			columnName: String,
			type: DT,
			toTable: Table<ttUE, *>,
			onDelete: ForeignRowReference.OnDelete,
		): ForeignRowReference<tUE, ttUE, RT, DT>
	}
}